package nl.saxion.oop.week7.model;

import nl.saxion.app.SaxionApp;
import nl.saxion.oop.week7.model.pieces.*;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collection;

public class Board
{
    private static final Color LIGHT = Color.lightGray;
    private static final Color DARK = Color.darkGray;

    public static final int MARGIN = 20;
    public static final int SIZE = 80;
    private final Collection<ChessPiece> pieces = new ArrayList<>();

    private int sx = -1,sy = -1;
    private ChessPiece selected;
    private final Collection<Move> moves = new ArrayList<>();

    private boolean isDarkSide = false;

    public Board()
    {
        for (int i = 0; i < 8; i++)
        {
            pieces.add(new Pawn(false, i));
            pieces.add(new Pawn(true, i));
        }
        pieces.add(new Rook(false, 0));
        pieces.add(new Rook(true, 0));
        pieces.add(new Rook(false, 7));
        pieces.add(new Rook(true, 7));
        pieces.add(new Knight(false, 1));
        pieces.add(new Knight(true, 1));
        pieces.add(new Knight(false, 6));
        pieces.add(new Knight(true, 6));
        pieces.add(new Bishop(false, 2));
        pieces.add(new Bishop(true, 2));
        pieces.add(new Bishop(false, 5));
        pieces.add(new Bishop(true, 5));
        pieces.add(new King(false, 4));
        pieces.add(new King(true, 4));
        pieces.add(new Queen(false, 3));
        pieces.add(new Queen(true, 3));
    }

    public void draw()
    {
        drawBoard();
        for (ChessPiece piece:pieces)
        {
            piece.drawPiece();
        }
        if (selected!=null)
        {
            this.moves.clear();
            Collection<Move> candidates = selected.getMoves();
            for (Move move:candidates)
            {
                // Check if move is legal.
                if (isMoveLegal(move))
                {
                    this.moves.add(move);
                    // Draw (for instance) a green circle.
                    if (move.takes())
                    {
                        SaxionApp.setFill(Color.RED);
                    }
                    else
                    {
                        SaxionApp.setFill(Color.GREEN);
                    }
                    SaxionApp.drawCircle(MARGIN + move.getToX() * SIZE + 40, MARGIN + move.getToY() * SIZE + 40, 25);
                }
            }
        }
    }

    /**
     * You don't want to edit this function... Trust me...
     * @param move
     * @return
     */
    private boolean isMoveLegal(Move move)
    {
        ChessPiece piece = getPieceAt(move.getToX(), move.getToY());
        // If the piece at the end position is on our side...
        if (piece!=null)
        {
            if (piece.isDark()==selected.isDark())
            {
                return false;
            }
            else
            {
                move.setTakes();
            }
        }
        if (selected.getName().equals("knight")) // Guess why I didn't use "instanceof".
        {
            return true;
        }
        else if (selected.getName().equals("pawn")) // Guess why I didn't use "instanceof".
        {
            if (move.getToX()==move.getFromX())
            {
                return piece==null;
            }
            else
            {
                return piece!=null;
            }
        }
        else
        {
            int dx = move.getToX()-move.getFromX();
            int dy = move.getToY()-move.getFromY();
            // Math magic to convert sx (and sy) to either -1, 0 or 1.
            int sx = (dx>0)?1:((dx<0)?-1:0);
            int sy = (dy>0)?1:((dy<0)?-1:0);
            int steps = Math.max(Math.abs(dx), Math.abs(dy));
            for (int i = 1; i < steps; i++)
            {
                if (getPieceAt(move.getFromX()+sx*i, move.getFromY()+sy*i)!=null)
                {
                    return false;
                }
            }
            return true;
        }
//        return true;
    }

    private void drawBoard()
    {
//        SaxionApp.clear();
        Color c = LIGHT;
        for (int y = 0; y < 8; y++)
        {
            for (int x = 0; x < 8; x++)
            {
                SaxionApp.setFill(c);

                if (x==sx && y==sy) {
                    SaxionApp.setBorderColor(Color.YELLOW);
                }
                else
                {
                    SaxionApp.setBorderColor(Color.BLACK);
                }

                SaxionApp.drawRectangle(x*SIZE+MARGIN, y*SIZE+MARGIN, SIZE-1, SIZE-1);
                if (x<7)
                {
                    // Switch color.
                    if (c == LIGHT)
                    {
                        c = DARK;
                    }
                    else
                    {
                        c = LIGHT;
                    }
                }
            }
        }
    }

    private ChessPiece getPieceAt(int x, int y)
    {
        for (ChessPiece piece:pieces)
        {
            if (piece.getX()==x && piece.getY()==y)
            {
                return piece;
            }
        }
        return null;
    }

    private Move getMoveTo(int x, int y)
    {
        for (Move move:moves)
        {
            if (move.getToX()==x && move.getToY()==y)
            {
                return move;
            }
        }
        return null;
    }

    public void doMove(int mx, int my)
    {
        // Coordinates to indicate the selected square.
        sx = (mx-MARGIN)/SIZE;
        sy = (my-MARGIN)/SIZE;

        Move m = getMoveTo(sx,sy);
        if (m!=null)
        {
            ChessPiece taken = getPieceAt(m.getToX(), m.getToY());
            if (taken!=null)
            {
                pieces.remove(taken);
            }
            selected.moveTo(m);
            isDarkSide = !isDarkSide;
            moves.clear();
            selected = null;
            return;
        }

        ChessPiece p = getPieceAt(sx,sy);
        if (p!=null && isDarkSide == p.isDark())
        {
            this.selected=p;
        }
    }
}
