package nl.saxion.oop.week7.model.pieces;

import nl.saxion.app.SaxionApp;
import nl.saxion.oop.week7.model.Move;

import java.util.Collection;

import static nl.saxion.oop.week7.model.Board.MARGIN;
import static nl.saxion.oop.week7.model.Board.SIZE;

public abstract class ChessPiece
{
    private final String name;
    private final boolean isDark;
    private int x;
    private int y;

    private boolean moved;

    public ChessPiece(String name, boolean isDark, int x, int y)
    {
        this.name = name;
        this.isDark = isDark;
        this.x = x;
        this.y = y;
        this.moved = false;
    }

    public void drawPiece()
    {
        String filename = name + "_" + (isDark ? "dark" : "light") + ".png";
        SaxionApp.drawImage("Exercise6/images/" + filename, 10 + x * SIZE + MARGIN, 10 + y * SIZE + MARGIN, 60, 60);
    }

    public String getName()
    {
        return name;
    }

    public boolean hasMoved()
    {
        return moved;
    }

    public boolean isDark()
    {
        return isDark;
    }

    public void moveTo(Move move)
    {
        this.moved = true;
        this.x = move.getToX();
        this.y = move.getToY();
    }

    public int getX()
    {
        return x;
    }

    public int getY()
    {
        return y;
    }

    /**
     * Returns all possible moves this piece can take.
     * Please only take the movement into account, not the lagality of the move.
     * For instance a pawn that hasn't moved can move to four other positions.
     * Once the pawn has moved only three.
     * @return a collection of {@link Move} objects.
     */
    public abstract Collection<Move> getMoves();

    /**
     * These functions are provided to check that your piece remains on the board.
     * @param tx
     * @param ty
     * @return
     */
    protected static boolean isOnBoard(int tx, int ty)
    {
        return tx>=0 && tx<=7 && ty>=0 && ty<=7;
    }

    protected static void tryAddMove(Collection<Move> moves, int x, int y, int tx, int ty)
    {
        if (isOnBoard(tx,ty))
        {
            moves.add(new Move(x,y,tx,ty));
        }
    }
}
